@@ -10,7 +10,7 @@
     'Sys.setenv(ImageMagick='path2imagemagick')'")
   }
   dcraw <- Sys.which("dcraw")
-  if(!file.exists(dcraw)) cat("Reading RAW images requires to install dcraw, see \n
+  if(!file.exists(dcraw)) packageStartupMessage("Reading RAW images requires to install dcraw, see \n
     http://cybercom.net/~dcoffin/dcraw/ for LINUX and http://www.insflug.org/raw/ 
     for MAC OS and Windows \n")
   adimpro.options()
